﻿EnableExplicit

;ONLY XIncludeFILE THIS MAIN SOURCE FILE IF USING NEXUSII IN THE FORM OF A SOURCE CODE INCLUDE (INCLUDING CREATING A DLL FROM THE SOURCE).
;OTHERWISE, IF USING A DLL VERSION OF NEXUSII, THEN SIMPLY XIncludeFile "NexusII_HeaderFile.pbi" AND BYPASS THE MAIN SOURCE FILE COMPLETELY.

CompilerIf Defined(INCLUDE_NEXUSII, #PB_Constant)=0
#INCLUDE_NEXUSII = 1
;/////////////////////////////////////////////////////////////////////////////////
;***NEXUS II LIBRARY OF CUSTOM CONTROLS***
;
;©nxSoftware.com 2010.
;=====================
;   Stephen Rodriguez (srod)
;   Created with Purebasic 4.5 for Windows.
;
;   Platforms:  Windows 2000 and above. Compile with PB 4.5 (and above) x86 or x64.
;               No promises with earlier versions of Windows!
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;NOTES.
;
;   i)    Purebasic applications using NexusII as a source code include would XIncludeFile just the file "NexusII.pbi" (this source file).
;
;         The next thing you have to do is decide which controls to add to the compilation?
;         You do this by simply defining the appropriate constants (from the following list) in your
;         source :
; 
;           #INCLUDE_NEXUSII_SPLITTER = 1
;
;         (Uncommenting the above lines will include all controls in all applications using this source etc.)
;         
;   ii)   NexusII exposes an OOP interface with a single class representing every kind of control. When creating a new control (e.g. a splitter),
;         the function responsible for creating the control instantiates a NexusIIControl object and returns a pointer to it to the calling function.
;         See the NexusII header file for a listing of the class methods etc.
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-NexusII constants / prototypes / structures.

  ;The following enumeration is for convenience only and are for use in vTable data-sections to avoid stuffing lots of zero's
  ;representing unimplemented class methods.
  ;The use of any of the following constants in a vTable data-section means that the underlying method is not implemented for the particular control.
    Enumeration
      #NEXUSII_DESTROY_NOTIMP              = 0
      #NEXUSII_GETCONTROLTYPE_NOTIMP       = 0
      #NEXUSII_ADDCOLUMN_NOTIMP            = 0
      #NEXUSII_ADDITEM_NOTIMP              = 0
      #NEXUSII_CLEARALLITEMS_NOTIMP        = 0
      #NEXUSII_CLOSEGADGETLIST_NOTIMP      = 0
      #NEXUSII_DISABLE_NOTIMP              = 0
      #NEXUSII_GETATTRIBUTE_NOTIMP         = 0
      #NEXUSII_GETCOLOR_NOTIMP             = 0
      #NEXUSII_GETDATA_NOTIMP              = 0
      #NEXUSII_GETHEIGHT_NOTIMP            = 0
      #NEXUSII_GETHWND_NOTIMP              = 0
      #NEXUSII_GETID_NOTIMP                = 0
      #NEXUSII_GETITEMATTRIBUTE_NOTIMP     = 0
      #NEXUSII_GETITEMCOLOR_NOTIMP         = 0
      #NEXUSII_GETITEMDATA_NOTIMP          = 0
      #NEXUSII_GETITEMSTATE_NOTIMP         = 0
      #NEXUSII_GETITEMTEXT_NOTIMP          = 0
      #NEXUSII_GETNUMBEROFITEMS_NOTIMP     = 0
      #NEXUSII_GETSTATE_NOTIMP             = 0
      #NEXUSII_GETTEXT_NOTIMP              = 0
      #NEXUSII_GETWIDTH_NOTIMP             = 0
      #NEXUSII_GETX_NOTIMP                 = 0
      #NEXUSII_GETY_NOTIMP                 = 0
      #NEXUSII_HIDE_NOTIMP                 = 0
      #NEXUSII_OPENGADGETLIST_NOTIMP       = 0
      #NEXUSII_REMOVECOLUMN_NOTIMP         = 0
      #NEXUSII_REMOVEITEM_NOTIMP           = 0
      #NEXUSII_RESIZE_NOTIMP               = 0
      #NEXUSII_SETATTRIBUTE_NOTIMP         = 0
      #NEXUSII_SETCOLOR_NOTIMP             = 0
      #NEXUSII_SETDATA_NOTIMP              = 0
      #NEXUSII_SETFONT_NOTIMP              = 0
      #NEXUSII_SETITEMATTRIBUTE_NOTIMP     = 0
      #NEXUSII_SETITEMCOLOR_NOTIMP         = 0
      #NEXUSII_SETITEMDATA_NOTIMP          = 0
      #NEXUSII_SETITEMSTATE_NOTIMP         = 0
      #NEXUSII_SETITEMTEXT_NOTIMP          = 0
      #NEXUSII_SETSTATE_NOTIMP             = 0
      #NEXUSII_SETTEXT_NOTIMP              = 0
      #NEXUSII_SENDMESSAGE_NOTIMP          = 0
    EndEnumeration


  #_NEXUSII_Prop$ = "NexusIIControlObject_PROP"

  ;The following enumeration indexes icons within our global state image-list. The first icon (index 0) will be a checked checkbox icon etc.
  ;We store copies of all relevant icons as an alternative to using the theming api's etc.
    Enumeration
        #NEXUSII_STATEICON_CHECKED
        #NEXUSII_STATEICON_UNCHECKED
        #NEXUSII_STATEICON_CLOSED         ;(Tree gadget close icon).
        #NEXUSII_STATEICON_OPEN           ;(Tree gadget open icon).
      #NEXUSII_STATEICON_ENDOFICONS
    EndEnumeration

  XIncludeFile "NexusII_HeaderFile.pbi"  ;Public constants etc.

  ;All controls offering a callback must utilise the following prototype for the callback function.
  ;The Control.NexusIIControl interface is declared within the NexusII header file.
    Prototype.i _NexusIICALLBACK(Control.NexusIIControl, uMsg, wParam=0, lParam=0, xParam=0)

  ;The following structure holds all 'global' variables.
    Structure _NexusIIGlobals
      hStateImageList.i         ;This image list contains the icons listed in the above enumeration which are added by the NexusII_Init() function.
                                ;The icons reside within the data section below.
                                ;For use by any control requiring such icons.
      cursorArrow.i
      cursorHand.i
      cursorSizeNS.i
      cursorSizeWE.i
    EndStructure

  ;The following is the generic NexusIIControl class template which all controls should extend.
    Structure _NexusII_GenericTemplate
      *vTable.i
      styles.i
      callback._NexusIICALLBACK
      hWnd.i
      ctrlID.i
    EndStructure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-Globals.
  Global gNexusII._NexusIIGlobals
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-Nexus II code modules.
  XIncludeFile "Utilities\NexusII_CommonCode.pbi"

  CompilerIf Defined(INCLUDE_NEXUSII_SPLITTER, #PB_Constant)
      XIncludeFile "Splitter\Splitter.pbi"
  CompilerEndIf
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-Resources.
  DataSection 
    ;The following section contains the required icons. Cannot pack them because of the different algorithms used by PB x86 and PB x64.
      nexusII_Icons_Start:
        ;Checked icon.
          Data.l 1150
          Data.b 0, 0, 1, 0, 1, 0, 16, 16, 0, 0, 1, 0, 32, 0, 104, 4, 0, 0, 22, 0, 0, 0, 40, 0, 0, 0, 16, 0, 0, 0, 32, 0, 0, 0, 1, 0, 32, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 245, 245, 245, 255, 249, 249
          Data.b 249, 255, 248, 248, 248, 255, 245, 245, 245, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 204, 203, 202, 255, 219, 218, 218, 255, 233, 226, 223, 255, 186, 153, 140, 255, 189, 157, 144, 255, 246, 243, 242, 255, 237, 237, 236, 255, 236, 235
          Data.b 235, 255, 234, 233, 233, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 202, 200, 198, 255, 240, 236
          Data.b 234, 255, 187, 153, 139, 255, 151, 95, 74, 255, 152, 97, 76, 255, 209, 185, 176, 255, 249, 249, 249, 255, 246, 246, 246, 255, 230, 230, 230, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 209, 207, 205, 255, 233, 225, 222, 255, 149, 93, 72, 255, 150, 95, 73, 255, 151, 96, 75, 255, 164, 115
          Data.b 97, 255, 250, 249, 248, 255, 244, 244, 244, 255, 226, 226, 225, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244
          Data.b 244, 255, 225, 224, 222, 255, 170, 127, 110, 255, 148, 92, 71, 255, 226, 212, 207, 255, 167, 120, 103, 255, 151, 96, 75, 255, 213, 191, 183, 255, 246, 246, 246, 255, 222, 221, 220, 255, 244, 244
          Data.b 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 205, 201, 197, 255, 221, 207, 201, 255, 200, 174, 163, 255, 238, 238
          Data.b 237, 255, 213, 193, 186, 255, 150, 94, 73, 255, 165, 118, 100, 255, 248, 248, 248, 255, 214, 213, 213, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 221, 217, 213, 255, 229, 226, 223, 255, 220, 216, 213, 255, 244, 243, 242, 255, 161, 113, 94, 255, 148, 92, 71, 255, 214, 195
          Data.b 188, 255, 220, 220, 219, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 213, 207
          Data.b 203, 255, 213, 207, 203, 255, 214, 209, 205, 255, 230, 226, 224, 255, 207, 184, 175, 255, 146, 90, 69, 255, 165, 119, 101, 255, 232, 231, 231, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 213, 207, 203, 255, 213, 207, 203, 255, 213, 207, 203, 255, 214, 208, 204, 255, 241, 238
          Data.b 237, 255, 157, 106, 87, 255, 146, 90, 68, 255, 208, 191, 185, 255, 246, 246, 246, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244
          Data.b 244, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 208, 204, 201, 255, 192, 167, 157, 255, 171, 134, 119, 255, 228, 223, 220, 255, 245, 245
          Data.b 245, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244
          Data.b 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 248, 248, 248, 255, 249, 249, 249, 255, 246, 246, 246, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3
          Data.b 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 255, 255, 0, 0
        ;UnChecked icon.
          Data.l 1150
          Data.b 0, 0, 1, 0, 1, 0, 16, 16, 0, 0, 1, 0, 32, 0, 104, 4, 0, 0, 22, 0, 0, 0, 40, 0, 0, 0, 16, 0, 0, 0, 32, 0, 0, 0, 1, 0, 32, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244
          Data.b 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 204, 203, 202, 255, 213, 212, 212, 255, 220, 219, 219, 255, 225, 225, 224, 255, 231, 231, 230, 255, 235, 235, 234, 255, 236, 236, 235, 255, 236, 235
          Data.b 235, 255, 234, 233, 233, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 198, 196, 194, 255, 233, 233
          Data.b 233, 255, 237, 237, 237, 255, 240, 240, 240, 255, 244, 244, 244, 255, 246, 246, 246, 255, 246, 246, 246, 255, 246, 246, 246, 255, 230, 230, 230, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 194, 191, 188, 255, 229, 228, 227, 255, 233, 233, 233, 255, 237, 237, 237, 255, 242, 242, 242, 255, 244, 244
          Data.b 244, 255, 245, 245, 245, 255, 244, 244, 244, 255, 226, 226, 225, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244
          Data.b 244, 255, 191, 187, 184, 255, 225, 223, 221, 255, 229, 229, 228, 255, 234, 234, 234, 255, 239, 239, 239, 255, 242, 242, 242, 255, 242, 242, 242, 255, 242, 242, 242, 255, 222, 221, 220, 255, 244, 244
          Data.b 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 188, 183, 178, 255, 220, 216, 213, 255, 223, 220, 218, 255, 227, 225
          Data.b 224, 255, 232, 232, 232, 255, 236, 236, 236, 255, 237, 237, 237, 255, 237, 237, 237, 255, 214, 213, 212, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 215, 209, 205, 255, 217, 212, 208, 255, 219, 215, 212, 255, 223, 221, 219, 255, 227, 226, 225, 255, 230, 230, 229, 255, 232, 232
          Data.b 232, 255, 205, 205, 204, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 213, 207
          Data.b 203, 255, 213, 207, 203, 255, 214, 209, 205, 255, 218, 213, 210, 255, 222, 219, 216, 255, 225, 223, 221, 255, 228, 227, 226, 255, 200, 199, 198, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 185, 179, 174, 255, 213, 207, 203, 255, 213, 207, 203, 255, 213, 207, 203, 255, 213, 207, 203, 255, 216, 211
          Data.b 208, 255, 220, 216, 213, 255, 223, 221, 219, 255, 197, 195, 193, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244
          Data.b 244, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 185, 179, 174, 255, 186, 180, 175, 255, 189, 185, 180, 255, 193, 190, 187, 255, 244, 244
          Data.b 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 143, 143, 142, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244
          Data.b 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 244, 244, 244, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143
          Data.b 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 143, 143, 142, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3
          Data.b 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 128, 3, 0, 0, 255, 255, 0, 0
        ;Closed icon.
          Data.l 1150
          Data.b 0, 0, 1, 0, 1, 0, 16, 16, 0, 0, 1, 0, 32, 0, 104, 4, 0, 0, 22, 0, 0, 0, 40, 0, 0, 0, 16, 0, 0, 0, 32, 0, 0, 0, 1, 0, 32, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 188, 187, 186, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145
          Data.b 145, 255, 188, 187, 186, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 227, 227
          Data.b 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 114, 66, 41, 255, 227, 227
          Data.b 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 145, 145, 145, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 114, 66, 41, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 145, 145, 145, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 251, 251, 250, 255, 167, 99, 75, 255, 167, 99
          Data.b 75, 255, 167, 99, 75, 255, 167, 99, 75, 255, 167, 99, 75, 255, 251, 251, 250, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 114, 66, 41, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251
          Data.b 250, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 252, 252
          Data.b 252, 255, 252, 252, 252, 255, 252, 252, 252, 255, 114, 66, 41, 255, 252, 252, 252, 255, 252, 252, 252, 255, 252, 252, 252, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 252, 252, 252, 255, 252, 252, 252, 255, 252, 252, 252, 255, 252, 252, 252, 255, 252, 252
          Data.b 252, 255, 252, 252, 252, 255, 252, 252, 252, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 188, 187, 186, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 188, 187, 186, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15
          Data.b 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0
        ;Opened icon.
          Data.l 1150
          Data.b 0, 0, 1, 0, 1, 0, 16, 16, 0, 0, 1, 0, 32, 0, 104, 4, 0, 0, 22, 0, 0, 0, 40, 0, 0, 0, 16, 0, 0, 0, 32, 0, 0, 0, 1, 0, 32, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 188, 187, 186, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145
          Data.b 145, 255, 188, 187, 186, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 227, 227
          Data.b 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 227, 227
          Data.b 227, 255, 227, 227, 227, 255, 227, 227, 227, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 145, 145, 145, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 236, 237, 237, 255, 145, 145, 145, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 236, 237, 237, 255, 167, 99, 75, 255, 167, 99
          Data.b 75, 255, 167, 99, 75, 255, 167, 99, 75, 255, 167, 99, 75, 255, 236, 237, 237, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251
          Data.b 250, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 251, 251
          Data.b 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 145, 145, 145, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 251, 251
          Data.b 250, 255, 251, 251, 250, 255, 251, 251, 250, 255, 145, 145, 145, 255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 188, 187, 186, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 145, 145, 145, 255, 188, 187, 186, 255, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
          Data.b 0, 0, 0, 0, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15
          Data.b 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 224, 15, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0, 255, 255, 0, 0
        ;end of icons
          Data.l 0
    EndDataSection
;/////////////////////////////////////////////////////////////////////////////////
CompilerEndIf

DisableExplicit

; IDE Options = PureBasic 4.50 (Windows - x86)
; CursorPosition = 137
; FirstLine = 137
; EnableUnicode
; EnableThread