﻿;/////////////////////////////////////////////////////////////////////////////////
;***Nexus II - splitter control ***
;*=================================
;*
;*©nxSoftWare (www.nxSoftware.com) 2010.
;*======================================
;/////////////////////////////////////////////////////////////////////////////////

;-CONSTANTS (internal, not required by users of the library).

  #_NEXUSIISPLITTER_Margin           = 1           ;Gap between child gadget and slider.
  #_NEXUSIISPLITTER_WidthOfSlider    = 3
  #_NEXUSIISPLITTER_LengthOfGripper  = 30


;-STRUCTURES (internal, not required by users of the library).

;/////////////////////////////////////////////////////////////////////////////////
  ;The following is the NexusIIControl class template for the NexusII splitter control
    Structure _NexusIISplitter_ClassTemplate Extends _NexusII_GenericTemplate
      gad1.i
      gad1Min.i
      gad2.i
      gad2Min.i
      ratio.d       ;Used in the proportional sizing of the child gadgets when the main splitter is resized.
      splitterPos.i
      sliderColor.i
      anchoredSliderColor.i
      lengthOfGripper.i
      sliderWidth.i
      oldProc.i
      ;The following two fields are used in the processing of the gripper.
        blnAnchored.i       ;Used to indicate that the slider has been anchored through the gripper.
        gadgetToCollapse.i  ;1 or 2.
    EndStructure


  ;Used to hold some global variables concerned with dragging etc.
    Structure _NexusIISplitter_DragGlobals
      dragGadget.i
      dragWinID.i
      offset.i              ;Difference between mouse pos and slider pos at the time of drag instigation.
      clientSize.i          ;Saves continually calculating the client size throughout the drag.
      splitterScreenPos.i   ;Saves continually calculating the screen pos of the slider throughout the drag.
      state.i               ;Bit 1 is set if the gripper is being dragged.
      pos.i                 ;Used to determine if the splitter moved.
      sliderMoved.i
      oldProc.i
    EndStructure

;/////////////////////////////////////////////////////////////////////////////////


;-GLOBALS.

;/////////////////////////////////////////////////////////////////////////////////
  Global gNexusIISplitterDrag._NexusIISplitter_DragGlobals
;/////////////////////////////////////////////////////////////////////////////////


;-DECLARES.

;/////////////////////////////////////////////////////////////////////////////////
  Declare.i NexusIISplitter_mainCallback(hWnd, uMsg, wParam, lParam)

  Declare.i NexusIISplitter_DespatchCallbackMessage(*this._NexusIISplitter_ClassTemplate, uMsg, wParam=0, lParam=0, xParam=0)
  Declare.i NexusIISplitter_HitTest(*this._NexusIISplitter_ClassTemplate, *pt.POINT=0)
  Declare.i NexusIISplitter_IsChildGadget(*this._NexusIISplitter_ClassTemplate, id)
  Declare NexusIISplitter_PaintSlider(*this._NexusIISplitter_ClassTemplate, hdc, offSet, width)
  Declare NexusIISplitter_PositionGadgets(*this._NexusIISplitter_ClassTemplate, blnNewlyCreated=#False)
  Declare NexusIISplitter_ResetRatio(*this._NexusIISplitter_ClassTemplate)
  Declare.i NexusIISplitter_SetSplitterPos(*this._NexusIISplitter_ClassTemplate, state, blnResetRatio = 1)


;/////////////////////////////////////////////////////////////////////////////////


;-CONTROL CREATION FUNCTION (exported).

;/////////////////////////////////////////////////////////////////////////////////
;The following function creates a new instance of the NexusIIControl class representing a new splitter control.
;Returns a NexusIIControl interface pointer or zero if an error.
Procedure.i NexusII_CreateSplitter(x, y, width, height, gadget1, gadget2, sliderWidth, styles=0, callback=0)
  Protected *this._NexusIISplitter_ClassTemplate
  ;Prior to using NexusII, an application must call the NexusII_Init() function and check it's return is non-zero. 
  ;Since we can call this function safely as many times as we wish, the best place for the call is within our control creation functions.
  If NexusII_Init()
    ;Allocate enough memory for a _NexusIISplitter_ClassTemplate.
      *this = AllocateMemory(SizeOf(_NexusIISplitter_ClassTemplate))
    If *this
      *this\ctrlID = ContainerGadget(#PB_Any, x, y, width, height, #PB_Container_BorderLess)
      If *this\ctrlID
        CloseGadgetList()
        With *this
          \vTable = ?_NexusIISplitter_ClassVT
          \callback = callback
          \hWnd = GadgetID(\ctrlID)
          \styles = styles
          \gad1 = gadget1
          \gad2 = gadget2
          \ratio = 0.5
          \sliderColor = -1
          \anchoredSliderColor = \sliderColor 
          \lengthOfGripper = #_NEXUSIISPLITTER_LengthOfGripper
          \sliderWidth = sliderWidth
          If \sliderWidth <= 0
            \sliderWidth = #_NEXUSIISPLITTER_WidthOfSlider
          EndIf
          \gadgetToCollapse = 1
          If styles&#NEXUSIISPLITTER_VERTICAL
            \splitterPos = (width-\sliderWidth)>>1
          Else
            \splitterPos = (height-\sliderWidth)>>1
          EndIf
        EndWith
        ;Set the splitter to be the parent of the two gadgets supplied.
          If IsGadget(gadget1)
            SetParent_(GadgetID(gadget1), *this\hWnd)
          EndIf
          If IsGadget(gadget2)
            SetParent_(GadgetID(gadget2), *this\hWnd)
          EndIf
        ;Place the address of the *this structure variable in a window property of the splitter
        ;so that the window procedure can access the variable.
          SetProp_(*this\hWnd, #_NEXUSII_Prop$, *this)
        ;Subclass the splitter.
          *this\oldProc = SetWindowLongPtr_(*this\hWnd, #GWL_WNDPROC, @NexusIISplitter_mainCallback())
        ;Position the child gadgets.
          If width And height
            NexusIISplitter_PositionGadgets(*this, #True)
          EndIf
      Else
        FreeMemory(*this)
        *this = 0
      EndIf
    EndIf
  EndIf
  ProcedureReturn *this
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-Callbacks.

;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NexusIISplitter_DragProc(hWnd, uMsg, wParam, lParam)
  Protected result, ps.PAINTSTRUCT, rcClient.RECT
  Protected pen, oldPen, colorSlider, brush, oldBrush
  Select uMsg
    Case #WM_ERASEBKGND
      ;Here we perform some custom painting.
        GetClientRect_(hWnd, rcClient)
        pen = CreatePen_(#PS_SOLID|#PS_INSIDEFRAME, 1, 0)
        colorSlider = GetSysColor_(#COLOR_3DFACE)
        brush = CreateSolidBrush_(colorSlider)
        oldPen = SelectObject_(wParam, pen)
        oldBrush = SelectObject_(wParam, brush)
          Rectangle_(wParam, 0,0,rcClient\right,rcClient\bottom)
        SelectObject_(wParam, oldPen)
        SelectObject_(wParam, oldBrush)
        DeleteObject_(pen)
        DeleteObject_(brush)
      result=1

    Case #WM_PAINT
      BeginPaint_(hWnd, ps)
      EndPaint_(hWnd, ps)
      result = 0
      
    Default
      result = CallWindowProc_(gNexusIISplitterDrag\oldProc, hWnd, uMsg, wParam, lParam)
  EndSelect    
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NexusIISplitter_mainCallback(hWnd, uMsg, wParam, lParam)
  Protected result, *this._NexusIISplitter_ClassTemplate, t1, rcClient.RECT, rc.RECT, face3dColor, colorMargin, brushMargin
  Protected hdc, ps.PAINTSTRUCT, state, pt.POINT, hParent, dHwnd, pos.w, ratio.d, width, height
  *this = GetProp_(hWnd, #_NEXUSII_Prop$)
  Select uMsg
    Case #WM_NCDESTROY
      If *this
        t1 = *this\oldProc
      EndIf
      SetWindowLongPtr_(*this\hWnd, #GWL_WNDPROC, *this\oldProc)
      RemoveProp_(*this\hWnd, #_NEXUSII_Prop$)
      FreeMemory(*this)
      If t1
        result = CallWindowProc_(t1, hWnd, uMsg, wParam, lParam)
      Else
        result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
      EndIf
  
    Case #WM_ERASEBKGND
      If *this
        ;Here we erase parts of the client area not occupied by the slider itself. Removing erasing from those parts of the splitter
        ;occupied by the child gadgets is problematic because of theming.
          If *this\styles & #NEXUSIISPLITTER_DONOTERASECHILDRECTS = 0
            GetClientRect_(hWnd, rcClient)
            ;Colors.
              face3dColor = GetSysColor_(#COLOR_3DFACE)
              colorMargin = GetGadgetColor(*this\ctrlID, #PB_Gadget_BackColor) ;Same as #NEXUSII_BACKCOLOR.
              If colorMargin = -1
                colorMargin = face3dColor
              EndIf
            ;Brushes.
              brushMargin = CreateSolidBrush_(colorMargin)
              If *this\styles & #NEXUSIISPLITTER_VERTICAL
                SetRect_(rc, 0, 0, *this\splitterPos-#_NEXUSIISPLITTER_Margin, rcClient\bottom)
                FillRect_(wParam, rc, brushMargin)
                SetRect_(rc, *this\splitterPos+#_NEXUSIISPLITTER_Margin+*this\sliderWidth, 0, rcClient\right, rcClient\bottom)
                FillRect_(wParam, rc, brushMargin)
              Else ;Horizontal.
                SetRect_(rc, 0, 0, rcClient\right, *this\splitterPos-#_NEXUSIISPLITTER_Margin)
                FillRect_(wParam, rc, brushMargin)
                SetRect_(rc, 0, *this\splitterPos+#_NEXUSIISPLITTER_Margin+*this\sliderWidth, rcClient\right, rcClient\bottom)
                FillRect_(wParam, rc, brushMargin)
              EndIf            
            ;Tidy up.
              DeleteObject_(brushMargin)
          EndIf
        result = 1
      Else
        Goto NexusIISplitter_mainCallback_L1
      EndIf

    Case #WM_LBUTTONDOWN
      If *this
        result = CallWindowProc_(*this\oldproc, hWnd, uMsg, wParam, lParam)
        GetClientRect_(hWnd, rcClient)
        pt\x = lParam&$ffff
        pt\y = (lParam>>16)&$ffff
        state = NexusIISplitter_HitTest(*this, pt)
        gNexusIISplitterDrag\sliderMoved = #False
        gNexusIISplitterDrag\state = state
        If *this\styles & #NEXUSIISPLITTER_VERTICAL
          If state & 1
            If *this\splitterPos-#_NEXUSIISPLITTER_Margin >= *this\gad1Min And (rcClient\right - *this\splitterPos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin) >= *this\gad2Min
              ;Proceed with the slider drag.
              MapWindowPoints_(hWnd, 0, rcClient, 1)
              ;We need to iterate through the splitter's parents and alter the dimensions of the slider
              ;in the cases that the splitter extends beyond the parent etc.
              hParent = GetParent_(hWnd)
              While hParent
                GetClientRect_(hParent, rc)
                MapWindowPoints_(hParent, 0, rc, 1)
                If rc\top > rcClient\top
                  rcClient\top = rc\top
                EndIf
                If rc\top + rc\bottom < rcClient\top + rcClient\bottom
                  rcClient\bottom = rc\top + rc\bottom - rcClient\top
                EndIf
                hParent = GetParent_(hParent)
              Wend
              gNexusIISplitterDrag\dragWinID = OpenWindow(#PB_Any, rcClient\left+*this\splitterPos, rcClient\top, *this\sliderWidth, rcClient\bottom,"",#PB_Window_BorderLess|#PB_Window_NoGadgets, hWnd)
              If gNexusIISplitterDrag\dragWinID
                dHwnd = WindowID(gNexusIISplitterDrag\dragWinID)
                gNexusIISplitterDrag\dragGadget = hWnd
                gNexusIISplitterDrag\pos = pt\x
                gNexusIISplitterDrag\oldProc = SetWindowLongPtr_(dhWnd, #GWL_WNDPROC, @NexusIISplitter_DragProc())
                gNexusIISplitterDrag\offset = pt\x - *this\splitterPos
                gNexusIISplitterDrag\clientSize = rcClient\right
                gNexusIISplitterDrag\splitterScreenPos = rcClient\left
                InvalidateRect_(dhWnd,0,1)
                SetCapture_(hWnd)
                SetFocus_(hWnd)
              EndIf
            EndIf
          ElseIf state & 2
            Goto NexusIISplitter_gotoVAnchor
          EndIf
        Else ;Horizontal.
          If state & 1
            If *this\splitterPos-#_NEXUSIISPLITTER_Margin >= *this\gad1Min And (rcClient\bottom - *this\splitterPos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin) >= *this\gad2Min
              ;Proceed with the slider drag.
              MapWindowPoints_(hWnd, 0, rcClient, 1)
              ;We need to iterate through the splitter's parents and alter the dimensions of the slider
              ;in the cases that the splitter extends beyond the parent etc.
              hParent = GetParent_(hWnd)
              While hParent
                GetClientRect_(hParent, rc)
                MapWindowPoints_(hParent, 0, rc, 1)
                If rc\left > rcClient\left
                  rcClient\left = rc\left
                EndIf
                If rc\left + rc\right < rcClient\left + rcClient\right
                  rcClient\right = rc\left + rc\right - rcClient\left
                EndIf
                hParent = GetParent_(hParent)
              Wend
              gNexusIISplitterDrag\dragWinID = OpenWindow(#PB_Any, rcClient\left, rcClient\top + *this\splitterPos, rcClient\right, *this\sliderWidth,"",#PB_Window_BorderLess|#PB_Window_NoGadgets, hWnd)
              If gNexusIISplitterDrag\dragWinID
                dHwnd = WindowID(gNexusIISplitterDrag\dragWinID)
                gNexusIISplitterDrag\dragGadget = hWnd
                gNexusIISplitterDrag\pos = pt\y
                gNexusIISplitterDrag\oldProc = SetWindowLongPtr_(dhWnd, #GWL_WNDPROC,@NexusIISplitter_DragProc())
                gNexusIISplitterDrag\offset = pt\y - *this\splitterPos
                gNexusIISplitterDrag\clientSize = rcClient\bottom
                gNexusIISplitterDrag\splitterScreenPos = rcClient\top
                InvalidateRect_(dhWnd,0,1)
                SetCapture_(hWnd)
                SetFocus_(hWnd)
              EndIf
            EndIf
          ElseIf state & 2
            Goto NexusIISplitter_gotoHAnchor
          EndIf
        EndIf
      Else
        result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
      EndIf

    Case #WM_LBUTTONUP
      If gNexusIISplitterDrag\dragWinID
        dhWnd = WindowID(gNexusIISplitterDrag\dragWinID)
        SetWindowLongPtr_(dhWnd, #GWL_WNDPROC, gNexusIISplitterDrag\oldProc)
        CloseWindow(gNexusIISplitterDrag\dragWinID)
        gNexusIISplitterDrag\dragWinID=0
        ReleaseCapture_()
        gNexusIISplitterDrag\dragGadget = 0
        GetClientRect_(hWnd, rcClient)
        If *this\styles & #NEXUSIISPLITTER_VERTICAL
          If gNexusIISplitterDrag\state &2 And gNexusIISplitterDrag\sliderMoved = #False ;Gripper has been clicked and not moved.
            state = NexusIISplitter_HitTest(*this)
            If state&2 ;Mouse released over the gripper.
NexusIISplitter_gotoVAnchor:
              If *this\blnAnchored
                ratio = *this\Ratio * (rcClient\right - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
                state = ratio + #_NEXUSIISPLITTER_Margin
                *this\blnAnchored = #False
                If NexusIISplitter_SetSplitterPos(*this, state, 0) = 0
                  InvalidateRect_(hWnd, 0, 1)
                EndIf
                ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
                  NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
              Else ;Need to collapse one of the child gadgets.
                ;Send a #NEXUSIISPLITTER_GRIPPERCLICKED message to the users callback.
                  result = NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_GRIPPERCLICKED, *this\gadgetToCollapse)
                If result 
                  *this\blnAnchored = #True
                  If result = #NEXUSIISPLITTER_ANCHORFIRSTGADGET
                    *this\gadgetToCollapse = 1
                  ElseIf result = #NEXUSIISPLITTER_ANCHORSECONDGADGET
                    *this\gadgetToCollapse = 2
                  EndIf
                  Select *this\gadgetToCollapse
                    Case 1
                      *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
                      width = NexusIISplitter_IsChildGadget(*this, *this\gad1) 
                      If width And width = GetFocus_() And *this\gad1Min = 0
                        SetFocus_(hWnd)
                      EndIf
                    Case 2
                      *this\splitterPos = rcClient\right - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
                      width = NexusIISplitter_IsChildGadget(*this, *this\gad2) 
                      If width And width = GetFocus_() And *this\gad2Min = 0
                        SetFocus_(hWnd)
                      EndIf
                  EndSelect
                  *this\gadgetToCollapse = 3-*this\gadgetToCollapse
                  NexusIISplitter_PositionGadgets(*this)
                  InvalidateRect_(hWnd, 0, 1)
                  ;Send a #NEXUSIISPLITTER_SLIDERANCHORED message to the users callback.
                    NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_SLIDERANCHORED, 3-*this\gadgetToCollapse)
                EndIf
              EndIf
            EndIf
          ElseIf gNexusIISplitterDrag\sliderMoved
            If *this\blnAnchored 
              ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
                NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
            EndIf
            *this\blnAnchored = #False
            pos = lParam&$ffff - gNexusIISplitterDrag\offset
            If pos - #_NEXUSIISPLITTER_Margin < *this\gad1Min
              pos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
            ElseIf gNexusIISplitterDrag\clientSize - pos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin < *this\gad2Min
              pos = gNexusIISplitterDrag\clientSize - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
            EndIf
            If NexusIISplitter_SetSplitterPos(*this, pos, 1) = 0
              InvalidateRect_(hWnd, 0, 1)
            EndIf
          EndIf
        Else ;Horizontal.
          If gNexusIISplitterDrag\state &2 And gNexusIISplitterDrag\sliderMoved = #False ;Gripper has been clicked and not moved.
            state = NexusIISplitter_HitTest(*this)
            If state&2 ;Mouse released over the gripper.
NexusIISplitter_gotoHAnchor:
              If *this\blnAnchored
                ratio = *this\Ratio * (rcClient\bottom - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
                state = ratio + #_NEXUSIISPLITTER_Margin
                *this\blnAnchored = #False
                If NexusIISplitter_SetSplitterPos(*this, state, 0) = 0
                  InvalidateRect_(hWnd, 0, 1)
                EndIf
                ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
                  NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
              Else ;Need to collapse one of the child gadgets.
                ;Send a #NEXUSIISPLITTER_GRIPPERCLICKED message to the users callback.
                  result = NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_GRIPPERCLICKED, *this\gadgetToCollapse)
                If result 
                  *this\blnAnchored = #True
                  If result = #NEXUSIISPLITTER_ANCHORFIRSTGADGET
                    *this\gadgetToCollapse = 1
                  ElseIf result = #NEXUSIISPLITTER_ANCHORSECONDGADGET
                    *this\gadgetToCollapse = 2
                  EndIf
                  Select *this\gadgetToCollapse
                    Case 1
                      *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
                      width = NexusIISplitter_IsChildGadget(*this, *this\gad1) 
                      If width And width = GetFocus_() And *this\gad1Min = 0
                        SetFocus_(hWnd)
                      EndIf
                    Case 2
                      *this\splitterPos = rcClient\bottom - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
                      width = NexusIISplitter_IsChildGadget(*this, *this\gad2) 
                      If width And width = GetFocus_() And *this\gad2Min = 0
                        SetFocus_(hWnd)
                      EndIf
                  EndSelect
                  *this\gadgetToCollapse = 3-*this\gadgetToCollapse
                  NexusIISplitter_PositionGadgets(*this)
                  InvalidateRect_(hWnd, 0, 1)
                  ;Send a #NEXUSIISPLITTER_SLIDERANCHORED message to the users callback.
                    NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_SLIDERANCHORED, 3-*this\gadgetToCollapse)
                EndIf
              EndIf
            EndIf
          ElseIf gNexusIISplitterDrag\sliderMoved
            If *this\blnAnchored 
              ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
                NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
            EndIf
            *this\blnAnchored = #False
            pos = (lParam>>16)&$ffff - gNexusIISplitterDrag\offset
            If pos-#_NEXUSIISPLITTER_Margin < *this\gad1Min
              pos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
            ElseIf gNexusIISplitterDrag\clientSize - pos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin < *this\gad2Min
              pos = gNexusIISplitterDrag\clientSize - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
            EndIf
            If NexusIISplitter_SetSplitterPos(*this, pos,1) = 0
              InvalidateRect_(hWnd, 0, 1)
            EndIf
          EndIf
        EndIf
      EndIf
      Goto NexusIISplitter_mainCallback_L1

    Case #WM_MOUSEMOVE
      If gNexusIISplitterDrag\dragGadget = hWnd
        If *this\styles & #NEXUSIISPLITTER_VERTICAL
          pos = lParam&$ffff
          If pos <> gNexusIISplitterDrag\pos
            gNexusIISplitterDrag\sliderMoved = #True
          EndIf
          ;Decide if the slider requires repositioning etc.
            pos - gNexusIISplitterDrag\offset
            If pos - #_NEXUSIISPLITTER_Margin < *this\gad1Min
              pos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
            ElseIf gNexusIISplitterDrag\clientSize - pos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin < *this\gad2Min
              pos = gNexusIISplitterDrag\clientSize - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
            EndIf
            If gNexusIISplitterDrag\splitterScreenPos + pos <> WindowX(gNexusIISplitterDrag\dragWinID)
              ResizeWindow(gNexusIISplitterDrag\dragWinID, gNexusIISplitterDrag\splitterScreenPos + pos, #PB_Ignore, #PB_Ignore, #PB_Ignore)
            EndIf
        Else ;Horizontal.
          pos = (lParam>>16&$ffff)
          If pos <> gNexusIISplitterDrag\pos
            gNexusIISplitterDrag\sliderMoved = #True
          EndIf
          ;Decide if the slider requires repositioning etc.
            pos - gNexusIISplitterDrag\offset
            If pos - #_NEXUSIISPLITTER_Margin <*this\gad1Min
              pos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
            ElseIf gNexusIISplitterDrag\clientSize - pos - *this\sliderWidth - #_NEXUSIISPLITTER_Margin < *this\gad2Min
              pos = gNexusIISplitterDrag\clientSize - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
            EndIf
            If gNexusIISplitterDrag\splitterScreenPos + pos <> WindowY(gNexusIISplitterDrag\dragWinID)
              ResizeWindow(gNexusIISplitterDrag\dragWinID, #PB_Ignore, gNexusIISplitterDrag\splitterScreenPos + pos, #PB_Ignore, #PB_Ignore)
            EndIf
        EndIf
      Else
        Goto NexusIISplitter_mainCallback_L1
      EndIf

    Case #WM_PAINT
      hdc = BeginPaint_(hWnd, ps)
      If hdc
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_CUSTOMPAINT, hdc, ps)
        GetClientRect_(hWnd, rcClient)
        ;Color.s
          face3dColor = GetSysColor_(#COLOR_3DFACE)
          colorMargin = GetGadgetColor(*this\ctrlID, #PB_Gadget_BackColor); Same as #NEXUSII_BACKCOLOR.
          If colorMargin = -1
            colorMargin = face3dColor
          EndIf
        ;Create brushes etc.
          brushMargin = CreateSolidBrush_(colorMargin)
          If *this\styles & #NEXUSIISPLITTER_VERTICAL
            ;First render the margins.
              ;Left.
                SetRect_(rc, *this\splitterPos-#_NEXUSIISPLITTER_Margin, 0, *this\splitterPos, rcClient\bottom)
                If rc\left<0 : rc\left = 0 : EndIf
                FillRect_(hdc, rc, brushMargin)
              ;Right.
                rc\left = *this\splitterPos + *this\sliderWidth
                rc\right = rc\left + #_NEXUSIISPLITTER_Margin
                FillRect_(hdc, rc, brushMargin)
            ;Now the splitter itself.
              NexusIISplitter_PaintSlider(*this, hdc, *this\splitterPos, rcClient\bottom)
          Else ;Horizontal.
            ;First render the margins.
              ;Top.
                SetRect_(rc, 0, *this\splitterPos-#_NEXUSIISPLITTER_Margin, rcClient\right, *this\splitterPos)
                If rc\top<0 : rc\top = 0 : EndIf
                FillRect_(hdc, rc, brushMargin)
              ;Bottom.
                rc\top = *this\splitterPos + *this\sliderWidth
                rc\bottom = rc\top + #_NEXUSIISPLITTER_Margin
                FillRect_(hdc, rc, brushMargin)
            ;Now the splitter itself.
              NexusIISplitter_PaintSlider(*this, hdc, *this\splitterPos, rcClient\right)
          EndIf        
        ;Tidy up.
          DeleteObject_(brushMargin)
        EndPaint_(hWnd, ps)
      EndIf

    Case #WM_SETCURSOR
      If *this And wParam = *this\hWnd
        state = NexusIISplitter_HitTest(*this)
        If state&2
          SetCursor_(LoadCursor_(0,#IDC_HAND))
        ElseIf state&1 And *this\styles & #NEXUSIISPLITTER_GRIPPERWITHNODRAG = 0
          If *this\styles & #NEXUSIISPLITTER_VERTICAL
            SetCursor_(gNexusII\cursorSizeWE)
          Else
            SetCursor_(gNexusII\cursorSizeNS)
          EndIf
        Else
          Goto NexusIISplitter_mainCallback_L1
        EndIf
      Else
        Goto NexusIISplitter_mainCallback_L1
      EndIf

    Case #WM_SIZE
      If *this
        result = CallWindowProc_(*this\oldproc, hWnd, uMsg, wParam, lParam)
        If lParam
          GetClientRect_(hWnd, rcClient)
          If *this\styles & #NEXUSIISPLITTER_VERTICAL
            If *this\blnAnchored
              Select *this\gadgetToCollapse
                Case 2 ;This means that the first gadget is currently anchored. In which case do nothing!
                Case 1 ;This means that the second gadget is currently anchored.
                       ;In which case, ensure the second gadget remains fixed.
                  *this\splitterPos = rcClient\right - *this\sliderWidth - #_NEXUSIISPLITTER_Margin - *this\gad2Min
              EndSelect
            ElseIf *this\styles & #NEXUSIISPLITTER_FIRSTFIXED
              If NexusIISplitter_IsChildGadget(*this, *this\gad1)
                width = GadgetWidth(*this\gad1)
              EndIf
              If width < *this\gad1Min
                width = *this\gad1Min
              EndIf
              If rcClient\right - width - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1 < *this\gad2Min
                width = rcClient\right - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1
              EndIf
              If width < 0
                width = 0
              EndIf
              *this\splitterPos = width + #_NEXUSIISPLITTER_Margin
              NexusIISplitter_ResetRatio(*this)
            ElseIf *this\styles & #NEXUSIISPLITTER_SECONDFIXED
              If NexusIISplitter_IsChildGadget(*this, *this\gad2)
                width = GadgetWidth(*this\gad2)
              EndIf
              If width < *this\gad2Min
                width = *this\gad2Min
              EndIf
              If rcClient\right - width - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1 < *this\gad1Min
                width = rcClient\right - *this\gad1Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1
              EndIf
              If width < 0
                width = 0
              EndIf
              *this\splitterPos = rcClient\right - width - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
              NexusIISplitter_ResetRatio(*this)
            Else
              ;With neither of the gadgets being fixed, we alter their widths in proportion to 
              ;their existing widths.
              ratio = *this\ratio * (rcClient\right - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
              *this\splitterPos = ratio + #_NEXUSIISPLITTER_Margin
              ;Check the minimum sizes etc.
                If *this\Gad1Min >= *this\Gad2Min
                  If *this\splitterPos < *this\gad1Min + #_NEXUSIISPLITTER_Margin
                    *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
                  EndIf
                Else
                  If rcClient\right - *this\splitterPos - #_NEXUSIISPLITTER_Margin - *this\sliderWidth < *this\gad2Min
                    *this\splitterPos = rcClient\right - #_NEXUSIISPLITTER_Margin - *this\sliderWidth - *this\gad2Min
                  EndIf
                EndIf
            EndIf
          Else ;Horizontal.
            If *this\blnAnchored
              Select *this\gadgetToCollapse
                Case 2 ;This means that the first gadget is currently anchored. In which case do nothing!
                Case 1 ;This means that the second gadget is currently anchored.
                        ;In which case, ensure the second gadget remains fixed.
                  *this\splitterPos = rcClient\bottom - *this\sliderWidth - #_NEXUSIISPLITTER_Margin - *this\gad2Min
              EndSelect
            ElseIf *this\styles & #NEXUSIISPLITTER_FIRSTFIXED
              If NexusIISplitter_IsChildGadget(*this, *this\gad1)
                height = GadgetHeight(*this\gad1)
              EndIf
              If height < *this\gad1Min
                height = *this\gad1Min
              EndIf
              If rcClient\bottom - height - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1 < *this\gad2Min
                height = rcClient\bottom - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1 
              EndIf
              If height < 0
                height = 0
              EndIf
              *this\splitterPos = height + #_NEXUSIISPLITTER_Margin
              NexusIISplitter_ResetRatio(*this)
            ElseIf *this\styles & #NEXUSIISPLITTER_SECONDFIXED
              If NexusIISplitter_IsChildGadget(*this, *this\gad2)
                height = GadgetHeight(*this\gad2)
              EndIf
              If height < *this\gad2Min
                height = *this\gad2Min
              EndIf
              If rcClient\bottom - height - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1 < *this\gad1Min
                height = rcClient\right - *this\gad1Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1
              EndIf
              If height < 0
                height = 0
              EndIf
              *this\splitterPos = rcClient\bottom - height - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
              NexusIISplitter_ResetRatio(*this)
            Else
              ;With neither of the gadgets being fixed, we alter their heights in proportion to 
              ;their existing heights.
              ratio = *this\ratio * (rcClient\bottom - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
              *this\splitterPos = ratio + #_NEXUSIISPLITTER_Margin
              ;Check the minimum sizes etc.
                If *this\Gad1Min >= *this\Gad2Min
                  If *this\splitterPos < *this\gad1Min + #_NEXUSIISPLITTER_Margin 
                    *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
                  EndIf
                Else
                  If rcClient\bottom - *this\splitterPos - #_NEXUSIISPLITTER_Margin - *this\sliderWidth< *this\gad2Min
                    *this\splitterPos = rcClient\bottom - #_NEXUSIISPLITTER_Margin - *this\sliderWidth - *this\gad2Min
                  EndIf
                EndIf
            EndIf
          EndIf
          NexusIISplitter_PositionGadgets(*this)
          InvalidateRect_(hWnd, 0, 1)
        EndIf
      Else
        result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
      EndIf

    Default  
NexusIISplitter_mainCallback_L1:
      If *this
        result = CallWindowProc_(*this\oldProc, hWnd, uMsg, wParam, lParam)
      Else
        result = DefWindowProc_(hWnd, uMsg, wParam, lParam)
      EndIf
  EndSelect
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-INTERNAL FUNCTIONS.

;/////////////////////////////////////////////////////////////////////////////////
;The following function despatches callback messages to any user-defined callback function.
Procedure.i NexusIISplitter_DespatchCallbackMessage(*this._NexusIISplitter_ClassTemplate, uMsg, wParam=0, lParam=0, xParam=0)
  Protected result=#True
  If *this\callback
    result = *this\callback(*this, uMsg, wParam, lParam, xParam)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following function performs a hitest on the splitter.
;*pt is in client coordinates. If *pt = 0 then we calculate the client coords.
;Returns the following:   bit 0 is set if the cursor is over the slider and ready to drag (#NEXUSIISPLITTER_GRIPPERWITHNODRAG is not set)
;                         bit 1 is set if the cursor is over the gripper.
Procedure.i NexusIISplitter_HitTest(*this._NexusIISplitter_ClassTemplate, *pt.POINT=0)
  Protected result, pt.POINT, rcClient.RECT
  GetClientRect_(*this\hWnd, rcClient)
  If *pt = 0
    GetCursorPos_(pt)
    MapWindowPoints_(0, *this\hWnd, pt, 1)
  Else
    CopyMemory(*pt, pt, SizeOf(POINT))
  EndIf
  If *this\styles & #NEXUSIISPLITTER_VERTICAL
    If pt\x >= *this\splitterPos And pt\x <= *this\splitterPos + *this\sliderWidth
      If *this\styles & #NEXUSIISPLITTER_GRIPPERWITHNODRAG = 0
        result = 1 ;Bit 0.
      EndIf
      ;Now check the gripper.
        If *this\styles & (#NEXUSIISPLITTER_GRIPPERWITHDRAG|#NEXUSIISPLITTER_GRIPPERWITHNODRAG) And *this\lengthOfGripper
          rcClient\top = (rcClient\bottom - *this\lengthOfGripper)>>1
          rcClient\bottom = rcClient\top + *this\lengthOfGripper 
          If pt\y > = rcClient\top And pt\y <= rcClient\bottom
            result | 2 ; Bit 1.
          EndIf
        EndIf
    EndIf
  Else ;Horizontal.
    If pt\y >= *this\splitterPos And pt\y <= *this\splitterPos + *this\sliderWidth
      If *this\styles & #NEXUSIISPLITTER_GRIPPERWITHNODRAG = 0
        result = 1 ;Bit 0.
      EndIf
      ;Now check the gripper.
        If *this\styles & (#NEXUSIISPLITTER_GRIPPERWITHDRAG|#NEXUSIISPLITTER_GRIPPERWITHNODRAG) And *this\lengthOfGripper
          rcClient\left = (rcClient\right- *this\lengthOfGripper)>>1
          rcClient\right = rcClient\left + *this\lengthOfGripper 
          If pt\x > = rcClient\left And pt\x <= rcClient\right
            result | 2 ; Bit 1.
          EndIf
        EndIf
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following function checks whether the specified 'child' gadget still exists and is part of
;the splitter etc.
;Returns the handle of the gadget if so.
Procedure.i NexusIISplitter_IsChildGadget(*this._NexusIISplitter_ClassTemplate, id)
  Protected chWnd
  If IsGadget(id)
    chWnd = GadgetID(id)
    If *this\hWnd <> GetParent_(chWnd)
      chWnd = 0
    EndIf
  EndIf
  ProcedureReturn chWnd
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following procedure paints the splitter's slider (minus the margins) into the specified hdc.
;The 'offSet' parameter = the slider pos for the static slider and zero for the drag window.
;The 'width' parameter gives the total width / height of the slider.
Procedure NexusIISplitter_PaintSlider(*this._NexusIISplitter_ClassTemplate, hdc, offSet, width)
  Protected rc.RECT, brushSlider, brushGripper
  Protected face3dColor, colorSlider, colorGripper
  ;Colors.
    face3dColor = GetSysColor_(#COLOR_3DFACE)
    colorSlider = *this\SliderColor
    If *this\blnAnchored
      colorSlider = *this\anchoredSliderColor
    EndIf
    If colorSlider = -1
      colorSlider = face3dColor
    EndIf
    colorGripper = colorSlider
    If colorGripper = face3dColor Or colorGripper=#White
      colorGripper = 0
    EndIf
  ;Create brushes etc.
    brushSlider = CreateSolidBrush_(colorSlider)
    If *this\styles & #NEXUSIISPLITTER_VERTICAL
      brushGripper = CreateHatchBrush_(#HS_HORIZONTAL, colorGripper)
    Else
      brushGripper = CreateHatchBrush_(#HS_VERTICAL, colorGripper)
    EndIf

  If *this\styles & #NEXUSIISPLITTER_VERTICAL
    If *this\styles & (#NEXUSIISPLITTER_GRIPPERWITHDRAG|#NEXUSIISPLITTER_GRIPPERWITHNODRAG)
      SetRect_(rc, offSet, 0, offSet+*this\sliderWidth, (width - *this\lengthOfGripper)>>1)
        FillRect_(hdc, rc, brushSlider)
      rc\top = (width - *this\lengthOfGripper)>>1
      rc\bottom = rc\top + *this\lengthOfGripper
        FillRect_(hdc, rc, brushGripper)
      rc\top = rc\bottom
      rc\bottom = width
        FillRect_(hdc, rc, brushSlider)
    Else
      SetRect_(rc, offSet, 0, offSet+*this\sliderWidth, width)
      FillRect_(hdc, rc, brushSlider)
    EndIf
  Else ;Horizontal.
    If *this\styles & (#NEXUSIISPLITTER_GRIPPERWITHDRAG|#NEXUSIISPLITTER_GRIPPERWITHNODRAG)
      SetRect_(rc, 0, offSet, (width - *this\lengthOfGripper)>>1, offSet+*this\sliderWidth)
        FillRect_(hdc, rc, brushSlider)
      rc\left = (width - *this\lengthOfGripper)>>1
      rc\right = rc\left + *this\lengthOfGripper
        FillRect_(hdc, rc, brushGripper)
      rc\left = rc\right
      rc\right = width
        FillRect_(hdc, rc, brushSlider)
    Else
      SetRect_(rc, 0, offSet, width, offSet+*this\sliderWidth)
      FillRect_(hdc, rc, brushSlider)
    EndIf
  EndIf
  ;Tidy up.
    DeleteObject_(brushSlider)
    DeleteObject_(brushGripper)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following procedure positions and resizes the child gadgets following a resize etc.
Procedure NexusIISplitter_PositionGadgets(*this._NexusIISplitter_ClassTemplate, blnNewlyCreated=#False)
  Protected rc.rect, hWnd, width1, width2, height1, height2, t1
  GetClientRect_(*this\hWnd, rc)
  If *this\styles&#NEXUSIISPLITTER_VERTICAL
    width1 = *this\splitterPos - #_NEXUSIISPLITTER_Margin
    If width1 < 0 : width1 = 0 : EndIf
    If NexusIISplitter_IsChildGadget(*this, *this\gad1)
      t1 = GadgetType(*this\gad1)
      If blnNewlyCreated Or t1 = #PB_GadgetType_ComboBox Or t1 = #PB_GadgetType_Panel
        ResizeGadget(*this\gad1, 0, 0, width1, rc\bottom)
      Else
        MoveWindow_(GadgetID(*this\gad1), 0, 0, width1, rc\bottom, 0)
      EndIf
    EndIf
    width2 = rc\right - *this\splitterPos - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
    If width2 < 0 : width2 = 0 : EndIf
    If NexusIISplitter_IsChildGadget(*this, *this\gad2)
      t1 = GadgetType(*this\gad2)
      If blnNewlyCreated Or t1 = #PB_GadgetType_ComboBox Or t1 = #PB_GadgetType_Panel
        ResizeGadget(*this\gad2, *this\splitterPos + #_NEXUSIISPLITTER_Margin + *this\sliderWidth, 0, width2, rc\bottom)
      Else
        MoveWindow_(GadgetID(*this\gad2), *this\splitterPos + #_NEXUSIISPLITTER_Margin + *this\sliderWidth, 0, width2, rc\bottom, 0)
      EndIf
    EndIf
    NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_GADGETSRESIZED, width1, width2)
  Else ;Horizontal.
    height1 = *this\splitterPos - #_NEXUSIISPLITTER_Margin
    If height1 < 0 : height1 = 0 : EndIf
    If NexusIISplitter_IsChildGadget(*this, *this\gad1)
      t1 = GadgetType(*this\gad1)
      If blnNewlyCreated Or t1 = #PB_GadgetType_ComboBox Or t1 = #PB_GadgetType_Panel
        ResizeGadget(*this\gad1,  0, 0, rc\right, height1)
      Else
        MoveWindow_(GadgetID(*this\gad1),  0, 0, rc\right, height1, 0)
      EndIf
    EndIf
    height2 = rc\bottom - *this\splitterPos - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
    If height2 < 0 : height2 = 0 : EndIf
    If NexusIISplitter_IsChildGadget(*this, *this\gad2)
      t1 = GadgetType(*this\gad2)
      If blnNewlyCreated Or t1 = #PB_GadgetType_ComboBox Or t1 = #PB_GadgetType_Panel
        ResizeGadget(*this\gad2, 0, *this\splitterPos + #_NEXUSIISPLITTER_Margin + *this\sliderWidth, rc\right, height2)
      Else
        MoveWindow_(GadgetID(*this\gad2), 0, *this\splitterPos + #_NEXUSIISPLITTER_Margin + *this\sliderWidth, rc\right, height2, 0)
      EndIf
    EndIf
    NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_GADGETSRESIZED, height1, height2)
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following procedure resets the splitter ratio when required.
Procedure NexusIISplitter_ResetRatio(*this._NexusIISplitter_ClassTemplate)
  Protected rcClient.RECT, state
  GetClientRect_(*this\hWnd, rcClient)
  state = *this\splitterPos-#_NEXUSIISPLITTER_Margin
  If state<0 : state = 0 : EndIf
  If *this\styles & #NEXUSIISPLITTER_VERTICAL  ;A vertical splitter.
    rcClient\right - #_NEXUSIISPLITTER_Margin>>1 - *this\sliderWidth
    If rcClient\right
      *this\ratio = state/rcClient\right
    EndIf
  Else ;Horizontal.
    rcClient\bottom - #_NEXUSIISPLITTER_Margin>>1 - *this\sliderWidth
    If rcClient\bottom
      *this\ratio = state/rcClient\bottom
    EndIf
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following is called by \SetState() and by the gripper positioning routine.
;Returns 0 if the splitter pos wasn't reset.
Procedure.i NexusIISplitter_SetSplitterPos(*this._NexusIISplitter_ClassTemplate, state, blnResetRatio = 1)
  Protected result, newGad1Width, newGad2Width, newGad1Height, newGad2Height
  Protected rcClient.RECT
  If state >= 0
    ;Here we reposition the slider.
      GetClientRect_(*this\hWnd, rcClient)
      If *this\styles&#NEXUSIISPLITTER_VERTICAL  ;A vertical splitter.
        newGad1Width = state - #_NEXUSIISPLITTER_Margin
        newGad2Width = rcClient\right - state - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        If newGad1Width < *this\gad1Min
        Debug 1
          state = *this\gad1Min+#_NEXUSIISPLITTER_Margin
        ElseIf newGad2Width < *this\gad2Min
          state = rcClient\right - *this\gad2Min - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        EndIf
        *this\splitterPos = state
        ;Reset the slider 'ratio'.
          If blnResetRatio
            NexusIISplitter_ResetRatio(*this)
          EndIf
        NexusIISplitter_PositionGadgets(*this)
        InvalidateRect_(*this\hWnd, 0, 1)
        result = #True
      Else ;Horizontal.
        newGad1Height = state - #_NEXUSIISPLITTER_Margin
        newGad2Height = rcClient\bottom - state - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        If newGad1Height < *this\gad1Min
          state = *this\gad1Min+#_NEXUSIISPLITTER_Margin
        ElseIf newGad2Height < *this\gad2Min
          state = rcClient\bottom - *this\gad2Min - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        EndIf
        *this\splitterPos = state
        ;Reset the slider 'ratio'.
          If blnResetRatio
            NexusIISplitter_ResetRatio(*this)
          EndIf
        NexusIISplitter_PositionGadgets(*this)
        InvalidateRect_(*this\hWnd, 0, 1)
        result = #True
    EndIf
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-NexusIIControl CLASS METHODS.

;/////////////////////////////////////////////////////////////////////////////////
Procedure NexusIISplitterClass_Destroy(*this._NexusIISplitter_ClassTemplate)
  FreeGadget(*this\ctrlID)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NexusIISplitterClass_GetControlType(*this._NexusIISplitter_ClassTemplate)
  ProcedureReturn #NEXUSII_SPLITTER
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure NexusIISplitterClass_Disable(*this._NexusIISplitter_ClassTemplate, state)
  DisableGadget(*this\ctrlID, state)
  If NexusIISplitter_IsChildGadget(*this, *this\gad1)
    DisableGadget(*this\gad1, state)
  EndIf
  If NexusIISplitter_IsChildGadget(*this, *this\gad2)
    DisableGadget(*this\gad2, state)
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NexusIISplitterClass_GetAttribute(*this._NexusIISplitter_ClassTemplate, attribute)
  Protected result
  Select attribute
    Case #NEXUSIISPLITTER_FIRSTMINIMUMSIZE
      result = *this\gad1Min
    Case #NEXUSIISPLITTER_SECONDMINIMUMSIZE
      result = *this\gad2Min
    Case #NEXUSIISPLITTER_FIRSTGADGET
      If NexusIISplitter_IsChildGadget(*this, *this\gad1)
        result = *this\gad1
      Else
        result = -1
      EndIf
    Case #NEXUSIISPLITTER_SECONDGADGET
      If NexusIISplitter_IsChildGadget(*this, *this\gad2)
        result = *this\gad2
      Else
        result = -1
      EndIf
    Case #NEXUSIISPLITTER_GRIPPERLENGTH
      result = *this\lengthOfGripper
    Case #NEXUSIISPLITTER_SLIDERTYPE
      result = *this\styles & (#NEXUSIISPLITTER_GRIPPERWITHDRAG | #NEXUSIISPLITTER_GRIPPERWITHNODRAG)
    Case #NEXUSIISPLITTER_SIZEOFSLIDERMARGIN
      result = #_NEXUSIISPLITTER_Margin
    Case #NEXUSIISPLITTER_SLIDERWIDTH
      result = *this\sliderWidth
  EndSelect
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i NexusIISplitterClass_GetColor(*this._NexusIISplitter_ClassTemplate, colorType)
  Select ColorType
    Case #NEXUSIISPLITTER_BACKCOLOR
      ProcedureReturn GetGadgetColor(*this\ctrlID, #PB_Gadget_BackColor)
    Case #NEXUSIISPLITTER_SLIDERCOLOR
      ProcedureReturn *this\sliderColor
    Case #NEXUSIISPLITTER_ANCHOREDSLIDERCOLOR
      ProcedureReturn *this\anchoredSliderColor
  EndSelect
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;The following gets the position of the slider or returns one of the TWO state flags to indicate that one of the gadgets is anchored etc.
;(#NEXUSIISPLITTER_ANCHORFIRSTGADGET or #NEXUSIISPLITTER_ANCHORSECONDGADGET).
Procedure.i NexusIISplitterClass_GetState(*this._NexusIISplitter_ClassTemplate)
  Protected result
  If *this\blnAnchored 
    result = *this\gadgetToCollapse - 4 ;This leaves one of the #NEXUSIISPLITTER_ANCHORFIRSTGADGET or #NEXUSIISPLITTER_ANCHORSECONDGADGET constants.
  Else
    ;Here we return the position of the slider which is the width of gad1 + margin.
      result = *this\splitterPos
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure NexusIISplitterClass_SetAttribute(*this._NexusIISplitter_ClassTemplate, attribute, value)
  Protected rcClient.RECT, hWnd
  Select attribute
    Case #NEXUSIISPLITTER_FIRSTMINIMUMSIZE
      *this\gad1Min = value
      If *this\blnAnchored And *this\gadgetToCollapse = 2
        *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
      EndIf
      ;Now arrange for the child controls to be resized.
        NexusIISplitter_SetSplitterPos(*this, *this\splitterPos, 0)
    Case #NEXUSIISPLITTER_SECONDMINIMUMSIZE
      *this\gad2Min = value
      If *this\blnAnchored And *this\gadgetToCollapse = 1
        GetClientRect_(*this\hWnd, rcClient)
        If *this\styles&#NEXUSIISPLITTER_VERTICAL
          *this\splitterPos = rcClient\right - *this\gad2Min - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        Else
          *this\splitterPos = rcClient\bottom - *this\gad2Min - #_NEXUSIISPLITTER_Margin - *this\sliderWidth
        EndIf
      EndIf
      ;Now arrange for the child controls to be resized.
        NexusIISplitter_SetSplitterPos(*this, *this\splitterPos, 0)
    Case #NEXUSIISPLITTER_FIRSTGADGET
      If IsGadget(value)
        ;Make the original gadget a child of the splitter parent.
          hWnd = NexusIISplitter_IsChildGadget(*this, *this\gad1)
          If hWnd And value <> *this\gad1
            SetParent_(hWnd, GetParent_(*this\hWnd))
          EndIf
        ;Make the new gadget a child of the splitter.
          If NexusIISplitter_IsChildGadget(*this, value) = 0
            SetParent_(GadgetID(value), *this\hWnd)
            *this\gad1 = value
            NexusIISplitter_PositionGadgets(*this)
          EndIf
      EndIf
    Case #NEXUSIISPLITTER_SECONDGADGET
      If IsGadget(value)
        ;Make the original gadget a child of the splitter parent.
          hWnd = NexusIISplitter_IsChildGadget(*this, *this\gad2)
          If hWnd And value <> *this\gad2
            SetParent_(hWnd, GetParent_(*this\hWnd))
          EndIf
        ;Make the new gadget a child of the splitter.
          If NexusIISplitter_IsChildGadget(*this, value) = 0
            SetParent_(GadgetID(value), *this\hWnd)
            *this\gad2 = value
            NexusIISplitter_PositionGadgets(*this)
          EndIf
      EndIf
    Case #NEXUSIISPLITTER_GRIPPERLENGTH
      *this\lengthOfGripper = value
      InvalidateRect_(*this\hWnd, 0, 1)
    Case #NEXUSIISPLITTER_SLIDERTYPE
      Select value
        Case 0  ;Remove the gripper.
          *this\styles&~(#NEXUSIISPLITTER_GRIPPERWITHDRAG | #NEXUSIISPLITTER_GRIPPERWITHNODRAG)
          InvalidateRect_(*this\hWnd, 0, 1)
        Case #NEXUSIISPLITTER_GRIPPERWITHNODRAG
          *this\styles|#NEXUSIISPLITTER_GRIPPERWITHNODRAG
          *this\styles&~#NEXUSIISPLITTER_GRIPPERWITHDRAG
          InvalidateRect_(*this\hWnd, 0, 1)
        Case #NEXUSIISPLITTER_GRIPPERWITHDRAG
          *this\styles|#NEXUSIISPLITTER_GRIPPERWITHDRAG
          *this\styles&~#NEXUSIISPLITTER_GRIPPERWITHNODRAG
          InvalidateRect_(*this\hWnd, 0, 1)
      EndSelect
  EndSelect
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure NexusIISplitterClass_SetColor(*this._NexusIISplitter_ClassTemplate, colorType, color)
  Select ColorType
    Case #NEXUSIISPLITTER_BACKCOLOR
      SetGadgetColor(*this\ctrlID, #PB_Gadget_BackColor, color)
    Case #NEXUSIISPLITTER_SLIDERCOLOR
      *this\SliderColor = color
      InvalidateRect_(*this\hWnd, 0, 1)
    Case #NEXUSIISPLITTER_ANCHOREDSLIDERCOLOR
      *this\anchoredSliderColor = color
      InvalidateRect_(*this\hWnd, 0, 1)
  EndSelect
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure NexusIISplitterClass_SetState(*this._NexusIISplitter_ClassTemplate, state)
  Protected width, rcClient.RECT, ratio.d
  If state = #NEXUSIISPLITTER_ANCHORFIRSTGADGET
    If *this\blnAnchored 
      ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
    EndIf
    *this\blnAnchored = #True
    *this\splitterPos = *this\gad1Min + #_NEXUSIISPLITTER_Margin
    width = NexusIISplitter_IsChildGadget(*this, *this\gad1) 
    If width And width = GetFocus_() And *this\gad1Min = 0
      SetFocus_(*this\hWnd)
    EndIf
    *this\gadgetToCollapse = 2
    NexusIISplitter_PositionGadgets(*this)
    InvalidateRect_(*this\hWnd, 0, 1)
    ;Send a #NEXUSIISPLITTER_SLIDERANCHORED message to the users callback.
      NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_SLIDERANCHORED, 1)
  ElseIf state = #NEXUSIISPLITTER_ANCHORSECONDGADGET
    If *this\blnAnchored 
      ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
    EndIf
    *this\blnAnchored = #True
    GetClientRect_(*this\hWnd, rcClient)
    If *this\styles & #NEXUSIISPLITTER_VERTICAL
      *this\splitterPos = rcClient\right - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
    Else
      *this\splitterPos = rcClient\bottom - *this\gad2Min - *this\sliderWidth - #_NEXUSIISPLITTER_Margin
    EndIf
    width = NexusIISplitter_IsChildGadget(*this, *this\gad2) 
    If width And width = GetFocus_() And *this\gad2Min = 0
      SetFocus_(*this\hWnd)
    EndIf
    *this\gadgetToCollapse = 1
    NexusIISplitter_PositionGadgets(*this)
    InvalidateRect_(*this\hWnd, 0, 1)
    ;Send a #NEXUSIISPLITTER_SLIDERANCHORED message to the users callback.
      NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_SLIDERANCHORED, 2)
  ElseIf state = #NEXUSIISPLITTER_RESTORESLIDER And *this\blnAnchored 
    GetClientRect_(*this\hWnd, rcClient)
    If *this\styles & #NEXUSIISPLITTER_VERTICAL
      ratio = *this\Ratio * (rcClient\right - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
      state = ratio + #_NEXUSIISPLITTER_Margin
      If NexusIISplitter_SetSplitterPos(*this, state, 0) = 0
        InvalidateRect_(*this\hWnd, 0, 1)
      EndIf
      ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
      *this\blnAnchored = #False
    Else ;Horizontal.
      ratio = *this\Ratio * (rcClient\bottom - *this\sliderWidth - #_NEXUSIISPLITTER_Margin>>1)
      state = ratio + #_NEXUSIISPLITTER_Margin
      If NexusIISplitter_SetSplitterPos(*this, state, 0) = 0
        InvalidateRect_(*this\hWnd, 0, 1)
      EndIf
      ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
      *this\blnAnchored = #False
    EndIf
  ElseIf state>=0
    If *this\blnAnchored 
      ;Send a #NEXUSIISPLITTER_ANCHORRELEASED message to the users callback.
        NexusIISplitter_DespatchCallbackMessage(*this, #NEXUSIISPLITTER_ANCHORRELEASED, 3-*this\gadgetToCollapse)
    EndIf
    *this\blnAnchored = #False
    NexusIISplitter_SetSplitterPos(*this, state)
  EndIf
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-DATA SECTION.

DataSection
  _NexusIISplitter_ClassVT:
    Data.i @NexusIISplitterClass_Destroy()
    Data.i @NexusIISplitterClass_GetControlType()
    Data.i #NEXUSII_ADDCOLUMN_NOTIMP
    Data.i #NEXUSII_ADDITEM_NOTIMP
    Data.i #NEXUSII_CLEARALLITEMS_NOTIMP
    Data.i #NEXUSII_CLOSEGADGETLIST_NOTIMP
    Data.i @NexusIISplitterClass_Disable()
    Data.i @NexusIISplitterClass_GetAttribute()
    Data.i @NexusIISplitterClass_GetColor()
    Data.i @NexusIIControlClass_GetData()
    Data.i @NexusIIControlClass_GetHeight()
    Data.i @NexusIIControlClass_GethWnd()
    Data.i @NexusIIControlClass_GetID()
    Data.i #NEXUSII_GETITEMATTRIBUTE_NOTIMP
    Data.i #NEXUSII_GETITEMCOLOR_NOTIMP
    Data.i #NEXUSII_GETITEMDATA_NOTIMP
    Data.i #NEXUSII_GETITEMSTATE_NOTIMP
    Data.i #NEXUSII_GETITEMTEXT_NOTIMP
    Data.i #NEXUSII_GETNUMBEROFITEMS_NOTIMP
    Data.i @NexusIISplitterClass_GetState()
    Data.i #NEXUSII_GETTEXT_NOTIMP
    Data.i @NexusIIControlClass_GetWidth()
    Data.i @NexusIIControlClass_GetX()
    Data.i @NexusIIControlClass_GetY()
    Data.i @NexusIIControlClass_Hide()
    Data.i #NEXUSII_OPENGADGETLIST_NOTIMP
    Data.i #NEXUSII_REMOVECOLUMN_NOTIMP
    Data.i #NEXUSII_REMOVEITEM_NOTIMP
    Data.i @NexusIIControlClass_Resize()
    Data.i @NexusIISplitterClass_SetAttribute()
    Data.i @NexusIISplitterClass_SetColor()
    Data.i @NexusIIControlClass_SetData()
    Data.i #NEXUSII_SETFONT_NOTIMP
    Data.i #NEXUSII_SETITEMATTRIBUTE_NOTIMP
    Data.i #NEXUSII_SETITEMCOLOR_NOTIMP
    Data.i #NEXUSII_SETITEMDATA_NOTIMP
    Data.i #NEXUSII_SETITEMSTATE_NOTIMP
    Data.i #NEXUSII_SETITEMTEXT_NOTIMP
    Data.i @NexusIISplitterClass_SetState()
    Data.i #NEXUSII_SETTEXT_NOTIMP
    Data.i #NEXUSII_SENDMESSAGE_NOTIMP
EndDataSection
; IDE Options = PureBasic 4.50 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 492
; FirstLine = 462
; Folding = ----
; Markers = 550
; EnableUnicode
; EnableThread
; Executable = d.exe.dll.exe.dll